/*
 * Copyright 2020-2023 XLAB d.o.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import { DateTime } from "luxon";

export function formatDateHelper(value) {
  if (value) {
    return DateTime.fromISO(value).toLocaleString({
      year: "numeric",
      month: "numeric",
      day: "numeric",
      hour: "numeric",
      minute: "numeric",
      second: "numeric",
    });
  }
}

export function getLastTree(historyData) {
  if (!historyData.length) return;
  let lastTreeIdx = 0;
  let closestTime = DateTime.fromISO(historyData[0].timeUpdated);
  for (let i = 0; i < historyData.length; i++) {
    let diff = DateTime.fromISO(historyData[i].timeUpdated).diff(
      DateTime.fromISO(closestTime),
    );
    if (diff.milliseconds > 0) {
      closestTime = historyData[i].timeUpdated;
      lastTreeIdx = i;
    }
  }
  return historyData[lastTreeIdx];
}

/** Function calls recursive function for rewriting tree JSON */
export function stringifyResources(data) {
  let dataNew = { children: [] };
  for (let i = 0; i < data.root.children.length; i++) {
    dataNew.children.push(findResourceType(data.root.children[i]));
  }
  return data;
}

/** Recursive function checks all the nodes, stringifies resourceType array and returns all others as they are */
function findResourceType(node) {
  if (!node || node.length === 0 || typeof node !== "object") return 0;
  let obj = {};
  const keys = Object.keys(node);
  keys.forEach((key) => {
    if (node[key] === null) {
      obj[key] = null;
    } else if (Array.isArray(node[key])) {
      if (key === "resourceType") {
        let resList = "[";
        for (let i = 0; i < node[key].length; i++) {
          resList += node[key][i];
          if (i < node[key].length - 1) {
            resList += ", ";
          }
        }
        node[key] = resList + "]";
      } else {
        obj[key] = node[key].map((x) => findResourceType(x));
      }
    } else if (typeof node[key] === "object") {
      obj[key] = findResourceType(node[key]);
    } else {
      obj[key] = node[key];
    }
  });
  return obj;
}
