/*
 * Copyright 2020-2023 XLAB d.o.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import { createApp } from "vue";
import App from "./App.vue";
import router from "./router";
import store from "@/store";
import axios from "axios";
import VueAxios from "vue-axios";
import Keycloak from "keycloak-js";

import "bootstrap/dist/css/bootstrap.css";
import "@/styles/common.scss";

import { dom } from "@fortawesome/fontawesome-svg-core";
dom.watch();
import "@fortawesome/fontawesome-free/css/all.css";
import client from "./client/client";

let initOptions = {
  url: process.env.VUE_APP_KEYCLOAK_URL,
  realm: "medina",
  clientId: "cce-frontend",
  onLoad: "login-required",
  checkLoginIframe: false,
};

let keycloak = new Keycloak(initOptions);

keycloak
  .init(initOptions)
  .then((auth) => {
    if (!auth) {
      window.location.reload();
    } else {
      client.defaults.headers.common.Authorization = "Bearer " + keycloak.token;

      const app = createApp(App);
      app.use(router);
      app.use(store);
      app.use(VueAxios, axios);
      app.mount("#app");
    }

    setInterval(() => {
      keycloak
        .updateToken(300)
        .then((refreshed) => {
          if (refreshed) {
            client.defaults.headers.common.Authorization =
              "Bearer " + keycloak.token;
          } else {
            console.warn(
              "Token not refreshed, valid for " +
                Math.round(
                  keycloak.tokenParsed.exp +
                    keycloak.timeSkew -
                    new Date().getTime() / 1000
                ) +
                " seconds"
            );
          }
        })
        .catch(() => {
          console.error("Failed to refresh token");
        });
    }, 300 * 1000);
  })
  .catch(() => {
    console.error("Keycloak authentication failed");
  });
