/*
 * Copyright 2020-2023 XLAB d.o.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

export const scaleTree = {
  methods: {
    getTreeRoot(scaleTreeRef) {
      return scaleTreeRef.treeChartCore.getNodeDataList()[0].data;
    },

    updateTree(scaleTreeRef) {
      scaleTreeRef.treeChartCore.updateDataset(this.getTreeRoot(scaleTreeRef));
      scaleTreeRef.nodeDataList = scaleTreeRef.treeChartCore.getNodeDataList();
    },

    collapseAll(scaleTreeRef) {
      this.collapseRecursively(this.getTreeRoot(scaleTreeRef));
      this.updateTree(scaleTreeRef);
    },

    expandAll(scaleTreeRef) {
      this.expandRecursively(this.getTreeRoot(scaleTreeRef));
      this.updateTree(scaleTreeRef);
    },

    collapseRecursively(node) {
      if (!node.children) {
        return;
      }

      node.children.forEach((i) => {
        this.collapseRecursively(i);
      });

      node._children = node.children;
      node.children = null;
      node._collapsed = true;
    },

    collapseAllFromLevel(node, level) {
      if (level <= 0) {
        this.collapseRecursively(node);
      } else {
        node.children?.forEach((i) => {
          this.collapseAllFromLevel(i, level - 1);
        });
      }
    },

    expandRecursively(node) {
      if (node._children) {
        node.children = node._children;
        node._children = null;
        node._collapsed = false;
      }
      node.children?.forEach((n) => {
        this.expandRecursively(n);
      });
    },

    hideNodeRecursive(node, condition) {
      if (condition(node)) {
        for (const key in node) {
          delete node[key];
        }
      } else {
        this.hideNodeChildrenRecursive(node, condition);
      }
    },

    hideNodeChildrenRecursive(node, condition) {
      let deleteIndex = 0;
      if (node._collapsed) {
        let length = node._children?.length > 0 ? node._children.length : 0;
        for (let i = 0; i < length; i++) {
          if (condition(node._children[deleteIndex])) {
            node._children.splice(deleteIndex, 1);
            continue;
          }
          this.hideNodeChildrenRecursive(
            node._children[deleteIndex],
            condition,
          );
          deleteIndex += 1;
        }
      } else {
        let length = node.children?.length > 0 ? node.children.length : 0;
        for (let i = 0; i < length; i++) {
          if (condition(node.children[deleteIndex])) {
            node.children.splice(deleteIndex, 1);
            continue;
          }
          this.hideNodeChildrenRecursive(node.children[deleteIndex], condition);
          deleteIndex += 1;
        }
      }
    },

    showNodeRecursive(originalNode, node, condition) {
      if (!originalNode || !condition(originalNode)) {
        node = {};
      } else if (!node?.code) {
        for (const key in originalNode) {
          if (key !== "children" && key !== "_children") {
            node[key] = originalNode[key];
          }
        }
        this.showNodeChildrenRecursive(originalNode, node, condition);
      } else {
        this.showNodeChildrenRecursive(originalNode, node, condition);
      }
    },

    showNodeChildrenRecursive(originalNode, node, condition) {
      let insertIndex = 0;
      if (originalNode._collapsed) {
        for (let i = 0; i < originalNode._children?.length; i++) {
          if (node?._collapsed) {
            if (
              node?._children?.length > insertIndex &&
              originalNode._children[i].code ===
                node._children[insertIndex].code
            ) {
              insertIndex += 1;
            } else if (condition(originalNode._children[i])) {
              if (!node?._children) {
                node._children = [];
                node._children.push(originalNode._children[i]);
              } else {
                node._children.splice(
                  insertIndex,
                  0,
                  originalNode._children[i],
                );
              }
              insertIndex += 1;
            }
            if (insertIndex) {
              this.showNodeChildrenRecursive(
                originalNode._children[i],
                node._children[insertIndex - 1],
                condition,
              );
            }
          } else {
            if (
              node?.children?.length > insertIndex &&
              originalNode._children[i].code === node.children[insertIndex].code
            ) {
              insertIndex += 1;
            } else if (condition(originalNode._children[i])) {
              if (!node?.children) {
                node.children = [];
                node.children.push(originalNode._children[i]);
              } else {
                node.children.splice(insertIndex, 0, originalNode._children[i]);
              }
              insertIndex += 1;
            }
            if (insertIndex) {
              this.showNodeChildrenRecursive(
                originalNode._children[i],
                node.children[insertIndex - 1],
                condition,
              );
            }
          }
        }
      } else {
        for (let i = 0; i < originalNode.children?.length; i++) {
          if (node?._collapsed) {
            if (
              node?._children?.length > insertIndex &&
              originalNode.children[i].code === node._children[insertIndex].code
            ) {
              insertIndex += 1;
            } else if (condition(originalNode.children[i])) {
              if (!node?._children) {
                node._children = [];
                node._children.push(originalNode.children[i]);
              } else {
                node._children.splice(insertIndex, 0, originalNode.children[i]);
              }
              insertIndex += 1;
            }
            if (insertIndex) {
              this.showNodeChildrenRecursive(
                originalNode.children[i],
                node._children[insertIndex - 1],
                condition,
              );
            }
          } else {
            if (
              node?.children?.length > insertIndex &&
              originalNode.children[i].code === node.children[insertIndex].code
            ) {
              insertIndex += 1;
            } else if (condition(originalNode.children[i])) {
              if (!node?.children) {
                node.children = [];
                node.children.push(originalNode.children[i]);
              } else {
                node.children.splice(insertIndex, 0, originalNode.children[i]);
              }
              insertIndex += 1;
            }
            if (insertIndex) {
              this.showNodeChildrenRecursive(
                originalNode.children[i],
                node.children[insertIndex - 1],
                condition,
              );
            }
          }
        }
      }
    },
  },
};
